"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .apikeyauthentication import APIKeyAuthentication, APIKeyAuthenticationTypedDict
from .basicauthauthentication import (
    BasicAuthAuthentication,
    BasicAuthAuthenticationTypedDict,
)
from .bearertokenauthentication import (
    BearerTokenAuthentication,
    BearerTokenAuthenticationTypedDict,
)
from .webhooksigningsecret import WebhookSigningSecret, WebhookSigningSecretTypedDict
from datetime import datetime
from typing import Dict, List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict
from zendesk.types import BaseModel


WebhookHTTPMethod = Literal["GET", "POST", "PUT", "PATCH", "DELETE"]
r"""HTTP method used for the webhook's requests. To subscribe the webhook to Zendesk events, this must be \"POST\" """

WebhookRequestFormat = Literal["json", "xml", "form_encoded"]
r"""The format of the data that the webhook will send. To subscribe the webhook to Zendesk events, this must be \"json\" """

WebhookStatus = Literal["active", "inactive"]
r"""Current status of the webhook"""

WebhookAuthenticationTypedDict = TypeAliasType(
    "WebhookAuthenticationTypedDict",
    Union[
        BasicAuthAuthenticationTypedDict,
        BearerTokenAuthenticationTypedDict,
        APIKeyAuthenticationTypedDict,
    ],
)
r"""Adds authentication to the webhook's HTTP requests, if not specified, the webhook will be unauthenticated"""


WebhookAuthentication = TypeAliasType(
    "WebhookAuthentication",
    Union[BasicAuthAuthentication, BearerTokenAuthentication, APIKeyAuthentication],
)
r"""Adds authentication to the webhook's HTTP requests, if not specified, the webhook will be unauthenticated"""


class WebhookExternalSourceTypedDict(TypedDict):
    r"""External source by which a webhook is created, e.g. Zendesk Marketplace"""

    name: NotRequired[str]
    r"""Name of the external source"""
    version: NotRequired[str]
    r"""Version of the external source"""


class WebhookExternalSource(BaseModel):
    r"""External source by which a webhook is created, e.g. Zendesk Marketplace"""

    name: Optional[str] = None
    r"""Name of the external source"""

    version: Optional[str] = None
    r"""Version of the external source"""


class WebhookTypedDict(TypedDict):
    name: str
    r"""Webhook name"""
    endpoint: str
    r"""The destination URL that the webhook notifies when Zendesk events occur"""
    http_method: WebhookHTTPMethod
    r"""HTTP method used for the webhook's requests. To subscribe the webhook to Zendesk events, this must be \"POST\" """
    request_format: WebhookRequestFormat
    r"""The format of the data that the webhook will send. To subscribe the webhook to Zendesk events, this must be \"json\" """
    status: WebhookStatus
    r"""Current status of the webhook"""
    id: NotRequired[str]
    r"""An auto-generated webhook id"""
    description: NotRequired[str]
    r"""Webhook description"""
    subscriptions: NotRequired[List[str]]
    r"""Event subscriptions for the webhook. To subscribe the webhook to Zendesk events, specify one or more event types.
    To connect the webhook to a trigger or automation, specify only \"conditional_ticket_events\" in the array

    """
    authentication: NotRequired[WebhookAuthenticationTypedDict]
    r"""Adds authentication to the webhook's HTTP requests, if not specified, the webhook will be unauthenticated"""
    custom_headers: NotRequired[Dict[str, str]]
    r"""Custom headers to deliver additional non-credential info to destinations"""
    signing_secret: NotRequired[WebhookSigningSecretTypedDict]
    r"""Signing secret used to verify webhook requests"""
    external_source: NotRequired[WebhookExternalSourceTypedDict]
    r"""External source by which a webhook is created, e.g. Zendesk Marketplace"""
    created_at: NotRequired[datetime]
    r"""When the webhook was created"""
    created_by: NotRequired[str]
    r"""id of the user who created the webhook. \"-1\" represents the Zendesk system"""
    updated_at: NotRequired[datetime]
    r"""When the webhook was last updated"""
    updated_by: NotRequired[str]
    r"""id of the user who last updated the webhook"""


class Webhook(BaseModel):
    name: str
    r"""Webhook name"""

    endpoint: str
    r"""The destination URL that the webhook notifies when Zendesk events occur"""

    http_method: WebhookHTTPMethod
    r"""HTTP method used for the webhook's requests. To subscribe the webhook to Zendesk events, this must be \"POST\" """

    request_format: WebhookRequestFormat
    r"""The format of the data that the webhook will send. To subscribe the webhook to Zendesk events, this must be \"json\" """

    status: WebhookStatus
    r"""Current status of the webhook"""

    id: Optional[str] = None
    r"""An auto-generated webhook id"""

    description: Optional[str] = None
    r"""Webhook description"""

    subscriptions: Optional[List[str]] = None
    r"""Event subscriptions for the webhook. To subscribe the webhook to Zendesk events, specify one or more event types.
    To connect the webhook to a trigger or automation, specify only \"conditional_ticket_events\" in the array

    """

    authentication: Optional[WebhookAuthentication] = None
    r"""Adds authentication to the webhook's HTTP requests, if not specified, the webhook will be unauthenticated"""

    custom_headers: Optional[Dict[str, str]] = None
    r"""Custom headers to deliver additional non-credential info to destinations"""

    signing_secret: Optional[WebhookSigningSecret] = None
    r"""Signing secret used to verify webhook requests"""

    external_source: Optional[WebhookExternalSource] = None
    r"""External source by which a webhook is created, e.g. Zendesk Marketplace"""

    created_at: Optional[datetime] = None
    r"""When the webhook was created"""

    created_by: Optional[str] = None
    r"""id of the user who created the webhook. \"-1\" represents the Zendesk system"""

    updated_at: Optional[datetime] = None
    r"""When the webhook was last updated"""

    updated_by: Optional[str] = None
    r"""id of the user who last updated the webhook"""
